﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using LukeSkywalker.IPNetwork;
using FileHelpers;

namespace SubnetOverlapTester
{
    public partial class Form1 : Form
    {
        public string csvFileLocation;
        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {

            try
            {

                this.btnBrowse.Enabled = false;
                this.txtCSVFilePath.Enabled = false;
                this.btnStart.Enabled = false;

                List<ADDMScanObj> scans = new List<ADDMScanObj>();

                FileHelperEngine fhEngine = new FileHelperEngine(typeof(SubnetData));
                SubnetData[] subnetDataList = fhEngine.ReadFile(csvFileLocation) as SubnetData[];

                this.txtProjectOutput.Text = "Loading subnets from file....\r\n";
                this.progressBar1.Minimum = 1;
                this.progressBar1.Step = 1;

                for (int i = 0; i < subnetDataList.Length; i++)
                {
                    string[] subnetsInScan = subnetDataList[i].Subnets.Split(',');
                    foreach (string subnetString in subnetsInScan)
                    {
                        ADDMScanObj tmpObj = new ADDMScanObj();
                        tmpObj.subnetName = subnetDataList[i].ScanName;
                        tmpObj.ipNetworkObj = LukeSkywalker.IPNetwork.IPNetwork.Parse(subnetString);
                        scans.Add(tmpObj);
                        tmpObj = null;
                    }
                }
                this.txtProjectOutput.Text += "Completed file load....\r\n";
                this.txtProjectOutput.Text += "Checking subnets for overlap, this will take a minute...\r\n";
                this.progressBar1.Maximum = scans.Count + (scans.Count/10);

                // now go through every network object and see if it overlaps with any other network objects
                foreach (ADDMScanObj scan in scans)
                {
                    // take this network and test it against all others in the list except itself
                    foreach (ADDMScanObj scantmp in scans)
                    {
                        if ((scan.subnetName == scantmp.subnetName) && (scan.ipNetworkObj == scantmp.ipNetworkObj))
                        {
                                // same exact scan and subnet, do nothing....
                        }else{
                            if (IPNetwork.Overlap(scan.ipNetworkObj, scantmp.ipNetworkObj))
                            {
                                this.txtProjectOutput.Text += "Scan " + scan.subnetName + " (" + scan.ipNetworkObj.Network.ToString() + "/" + scan.ipNetworkObj.Cidr.ToString() + ") overlaps with scan " + scantmp.subnetName + " (" + scantmp.ipNetworkObj.Network.ToString() + "/" + scantmp.ipNetworkObj.Cidr.ToString() + ")\r\n";
                            }
                        }
                    }

                    this.progressBar1.PerformStep();
                }

                scans = null;
                this.progressBar1.Value = this.progressBar1.Maximum;
                MessageBox.Show("Complete. See ouput window for subnet overlaps.");
            }
            finally
            {
                this.progressBar1.Value = 1;
                this.btnBrowse.Enabled = true;
                this.txtCSVFilePath.Enabled = true;
                this.btnStart.Enabled = true;
            }
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.Title = "Locate CSV Subnet File";
            fileDialog.InitialDirectory = @"c:\";
            fileDialog.Filter = "CSV Files (*.csv)|*.csv|All Files (*.*)|*.*";
            fileDialog.FilterIndex = 1;
            fileDialog.RestoreDirectory = false;
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                csvFileLocation = fileDialog.FileName;
                this.txtCSVFilePath.Text = fileDialog.FileName;
            }
        }
    }

    [IgnoreEmptyLines()]
    [DelimitedRecord(",")]
    public class SubnetData
    {
        public string ScanName;
        
        [FieldQuoted('"',QuoteMode.OptionalForRead,MultilineMode.NotAllow)]
        public string Subnets;
    }

    public class ADDMScanObj
    {
        public LukeSkywalker.IPNetwork.IPNetwork ipNetworkObj;
        public string subnetName;
    }
}
